﻿<#
.SYNOPSIS
  Vergleicht einen E-Voting Image USB Stick mit einem Referenz-Image und prüft ob sämtliche Dateien darauf vertrauenswürdig sind

.DESCRIPTION
  Prüft einen E-Voting Image USB Stick, in dem er ihn mit einem bekannten unmodifizierten Microsoft Windows ISO vergleicht,
  sowie Signaturen und Hash-Werte prüft.
  Dateien erfüllen die Prüfung, falls sie einer der folgenden drei Kriterien erfüllen: Sie sind identisch auf der originalen
  Microsoft ISO vorhanden, sie sind mit einem vertrauenswürdigen Zertifikat signiert, oder ihr Hash hat einen bekannten
  korrekten Wert.

.PARAMETER ReferenceISO
  Pfad zu einer unmodifizierten Windows LTSC 2021 Deutsch ISO Datei

.PARAMETER ImageUSB
  Laufwerksbuchstabe des USB Sticks, der geprüft werden soll

.PARAMETER DisplayPositiveResults
  Falls dieser Parameter gesetzt ist, werden auch erfolgreich geprüfte Dateien in der Konsole angezeigt

.INPUTS
  None

.OUTPUTS
  Eine Datei namens 'evoting_imagecheck.csv' wird im Dokumente-Ordner erstellt

.NOTES
  Version:        1.8
  Author:         Ontrex AG - Athman Boukhaoua
  Creation Date:  20.10.2022
  Purpose/Change: Script erstellt

.LINK  
  www.ontrex.ch
#>


param(
    [Parameter(Mandatory=$true)][string]$ReferenceISO,
    [Parameter(Mandatory=$true)][string]$ImageUSB,
    [switch]$DisplayPositiveResults = $false
)

# A list of certificate CNs we trust to sign executables
$trusted_signers = @(
    'CN=Ontrex AG, OU=CTO, O=Ontrex AG, L=Bruettisellen, S=Zuerich, C=CH'
    'CN="Open Source Developer, Dominik Reichl", O=Open Source Developer, L=Metzingen, S=Baden-Württemberg, C=DE'
    'CN=Microsoft Corporation, O=Microsoft Corporation, L=Redmond, S=Washington, C=US'
    'CN=Microsoft Windows, O=Microsoft Corporation, L=Redmond, S=Washington, C=US'
    'CN=.NET, O=Microsoft Corporation, L=Redmond, S=Washington, C=US'
    'CN=Microsoft Corporation, O=Microsoft Corporation, L=Redmond, S=Washington, C=US'
    'CN=Microsoft Windows Publisher, O=Microsoft Corporation, L=Redmond, S=Washington, C=US'
    'CN="Notepad++", O="Notepad++", L=Saint Cloud, S=Ile-de-France, C=FR'
    'CN=Ghisler Software GmbH, O=Ghisler Software GmbH, S=Bern, C=CH'
    'CN=A.E.T. Europe B.V., O=A.E.T. Europe B.V., L=Arnhem, S=Gelderland, C=NL'
    'CN=HP Inc., O=HP Inc., L=Palo Alto, S=California, C=US'
    'CN=HP Inc., OU=HP Cybersecurity, O=HP Inc., L=Palo Alto, S=California, C=US'
    'CN=Lenovo, OU=G10, O=Lenovo, L=Morrisville, S=North Carolina, C=US'
    'CN=Lenovo, OU=G11, O=Lenovo, L=Morrisville, S=North Carolina, C=US'
    'CN=Lenovo, O=Lenovo, L=Morrisville, S=North Carolina, C=US'
)

# A list of known hash values of files we have customized that aren't possible to sign
$trusted_hashes = @{
    'de-de_windows_10_enterprise_ltsc_2021_x64_dvd_71796d33.iso' = '710D8854127054B696152388C8527EF5D526ED5ECDFFF8B664F4C9EC809F6053'
    'autounattend.xml' = '63D1ED58C2D2A7A50B695972B8A456C3385DA7197A0F92A74A8BEE8E09A52532'
    'gmp.dll' = '74970AC1F9570B301D22788E62BF5D51CB885001A3CB630BA2C9DB798D5DEC30'
    '7z2409-x64.msi' = 'EC6AF1EA0367D16DDE6639A89A080A524CEBC4D4BEDFE00ED0CAC4B865A918D8'
    'openssl-3.4.0-win64.zip' = '3C139633D1E7D861FA55CCD944EBB60AD07F358574BB3F84BD162CE14E4CC885'
    'vmgj.zip' = '486DB645F8FF5E7223E37E412D5C6F2C45ECADD813EE14ADDBC93E3D8A142D67'
    'kse-553-setup.exe' = '4677123EF937847AC233E8F46A82B6E43854F60A728DCE16D85AA04FFC6D9762'
    'credentialguard.txt' = '9B2926AB458C37018AFC42158E087C42EDCAADE269B8E3CDDC13EEE41C0335F8'
    'ie_user.txt' = '93A751DDEEF902B3C300089BEC2F55F5DE8E67BDDF0FA1DB4F872962C6624036'
    'user.txt' = '1182AE66BDFE6796B3364DEC63DD2F82EC6DE19BE42E0B14B167390902350C16'
    'ie_computer.txt' = '961750F0FF1923BD084843C6771D0F212367BF1F19A1E9C70734E6212301B123'
    'defender.txt' = '016E13150DA8573CF55DDA319C4E7C14480921C6233E1078D46704FFC24983DD'
    'computer.txt' = '8D9277777FE7FD61C97A08A0BD14676977DE6D202067E64EAFAF243DD33E6B46'
    'bitlocker.txt' = '9B38454AA10CF67183C60980E8301917A3480D371C1B77A965872439AEFAC199'
    'edge.txt' = 'D942E516992B88A119093E3D23A1D77371B1822AFE211BF10290FF26968D1880'
    'cis.txt' = '6DFCFDF9FFC433D2E73FDF3097EF1F90764F0158C02518CBD47444777B806F05'
    'custom.txt' = 'F9D754E445A49AD0851EDBAE2EA4E8C2FA0FCDD8E345C041D46DCD1846DFE2FA'
    'code128.ttf' = '2998DE8D2EBED4F31BC7CE8F1A0414342D35068149A6A52C14BCE8EC8BCCB0DB'
    'OB______.TTF' = '82E335C05518655E045CE0584DF3A1B2F209EA43F3EE743AA82F48FE42FC1C05'
    'RobotoMono-Bold.ttf' = '3F4E0CD7747DF32A34748A66C1274D76712CAB429C75E75F5F0167319B97FCCB'
    'LayoutModification.xml' = '88D640375DC4C2122C32718A2EAD4EA05BAD9A8924F7AB833F92E86C6B562C6C'
    'timechangeforusers.inf' = 'CE632B7D6C65D15D3E398C180644E019ED83368C2DE84A2F4DF8F7FB71895CE9'
}


# Check if a file is signed, and if yes, whether the signing authority is in the list of trusted signers
function check-signature($file_name)
{
    # Get the signature of this file
    $signature = Get-AuthenticodeSignature $file_name

    # Verify if it's valid
    if ($signature.Status -eq [System.Management.Automation.SignatureStatus]::Valid) {
        # If it's valid, get the identity of the signer, and check if it's in our accepted list of signers
        $certificate = $signature.SignerCertificate
        if ($trusted_signers -contains $certificate.Subject) {
            # Only if the signature is valid and from an approved certificate, we return true
            return @{Result=$true; Signer=$certificate.Subject.Split(',')[0]}
        }
        else {
            Write-Host "$($certificate.Subject) is untrusted"
        }
    }

    # In any other case, return false
    return @{Result=$false; Signer="unsigned"}
}

# Calculate the hash of a file, and check if it matches a known trusted value
function check-hash($file_name)
{
    # Get the SHA-256 hash of the file
    $hash = (Get-FileHash $file_name).Hash

    # See if it's in our table of safe hashes
    if ($trusted_hashes[(Split-Path $file_name -Leaf)] -eq $hash) {
        # Return true if it is in there
        return @{Result=$true; Hash=$hash}
    }

    # And false if it couldn't be found, or wasn't equal
    return @{Result=$false; Hash=$hash}
}

# Mount an ISO file and return the drive letter it was mounted to
function mount-iso($path)
{
    # First find out which CD ROM drives already exist
    [array]$old_drives = (Get-Volume | ? DriveType -eq 'CD-ROM').DriveLetter

    # Then mount the ISO file and get another list of CD ROM drives
    $mount_info = Mount-DiskImage $ReferenceISO
    [array]$new_drives = (Get-Volume | ? DriveType -eq 'CD-ROM').DriveLetter

    # Then find out which CD ROM drive has just newly appeared and return that one's drive letter
    $r = ''
    $new_drives | % { if ($old_drives -notcontains $_) { $r = $_ } }

    $r
}

function process-result($filename, $result, $reason, $info)
{
    # Write a summary of the result to the console
    if ($result -eq 'ok') {
        if ($DisplayPositiveResults) {
            switch ($reason) {
                'original' {
                    Write-Host "$filename ok because original" -ForegroundColor Green
                }
                'signature' {
                    Write-Host "$filename ok by signature" -ForegroundColor Green
                }
                'hash' {
                    Write-Host "$filename ok by hash" -ForegroundColor Green
                }
            }
        }
    }
    else {
        Write-Host "$filename not ok" -ForegroundColor Red
    }

    # Format the results as an object so we can later output everything in a table
    New-Object PSObject -Property @{
        FileName = $filename
        Result = $result
        Reason = $reason
        Info = $info
    }
}





Write-Host "Verifying integrity of the reference ISO file"
$isofile = Get-FileHash $ReferenceISO
if ($isofile.Hash -ne $trusted_hashes['de-de_windows_10_enterprise_ltsc_2021_x64_dvd_71796d33.iso']) {
    Write-Host "Reference ISO file does not have the correct hash value" -ForegroundColor Red
    exit 1
}
Write-Host "  ISO file is ok"





Write-Host "Mounting reference ISO file"
$reference_drive = mount-iso $ReferenceISO

if (-not $reference_drive) {
    Write-Host "  Could not mount $ReferenceISO" -ForegroundColor Red
    exit 1
}
Write-Host "  Mounted to drive $reference_drive"




$verification_results = @()

ls $ImageUSB -File -Force -Recurse | % {
    $file_to_check = $_.FullName

    # Construct the name of the file to compare it with by removing the drive letter of the customized image
    # and instead adding the one from the original ISO
    $reference_file = $reference_drive + $file_to_check.Substring(1)

    if (Test-Path $reference_file) {
        # If this file also exists on the original Microsoft ISO image, then compare its hash with the original
        $reference_hash = Get-FileHash $reference_file
        $check_hash = Get-FileHash $file_to_check

        if ($check_hash.Hash -eq $reference_hash.Hash) {
            $verification_results += process-result $file_to_check "ok" "original" $check_hash.Hash
        }
        else {
            $verification_results += process-result $file_to_check "failed" "original" $check_hash.Hash
        }
    }
    else {
        # This file doesn't exist on the original ISO, so we have to check its signature

        $signature_check = check-signature $file_to_check

        if ($signature_check.Result) {
            $verification_results += process-result $file_to_check "ok" "signature" $signature_check.Signer
        }
        else {
            # If the file doesn't have a signature, then check the hash instead
            $hash_check = check-hash $file_to_check

            if ($hash_check.Result) {
                $verification_results += process-result $file_to_check "ok" "hash" $hash_check.Hash
            }
            else {
                $verification_results += process-result $file_to_check "failed" "hash" $hash_check.Hash
            }
        }


    }
}

$result_filename = Join-Path ([Environment]::GetFolderPath(“mydocuments”)) "evoting_imagecheck.csv"
$verification_results | Export-Csv $result_filename -NoTypeInformation -Encoding UTF8
Write-Host "Results of the scan have been written to:`n$result_filename"

Write-Host "Unmounting reference image"
Dismount-DiskImage $ReferenceISO | Out-Null

# SIG # Begin signature block
# MIIpHQYJKoZIhvcNAQcCoIIpDjCCKQoCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAuOyTilieKL7f6
# a1fkbbsQsqJKI5FdFb0uwxgD/fTjy6CCDiAwggawMIIEmKADAgECAhAIrUCyYNKc
# TJ9ezam9k67ZMA0GCSqGSIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNV
# BAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMTA0MjkwMDAwMDBaFw0z
# NjA0MjgyMzU5NTlaMGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwg
# SW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcg
# UlNBNDA5NiBTSEEzODQgMjAyMSBDQTEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAw
# ggIKAoICAQDVtC9C0CiteLdd1TlZG7GIQvUzjOs9gZdwxbvEhSYwn6SOaNhc9es0
# JAfhS0/TeEP0F9ce2vnS1WcaUk8OoVf8iJnBkcyBAz5NcCRks43iCH00fUyAVxJr
# Q5qZ8sU7H/Lvy0daE6ZMswEgJfMQ04uy+wjwiuCdCcBlp/qYgEk1hz1RGeiQIXhF
# LqGfLOEYwhrMxe6TSXBCMo/7xuoc82VokaJNTIIRSFJo3hC9FFdd6BgTZcV/sk+F
# LEikVoQ11vkunKoAFdE3/hoGlMJ8yOobMubKwvSnowMOdKWvObarYBLj6Na59zHh
# 3K3kGKDYwSNHR7OhD26jq22YBoMbt2pnLdK9RBqSEIGPsDsJ18ebMlrC/2pgVItJ
# wZPt4bRc4G/rJvmM1bL5OBDm6s6R9b7T+2+TYTRcvJNFKIM2KmYoX7BzzosmJQay
# g9Rc9hUZTO1i4F4z8ujo7AqnsAMrkbI2eb73rQgedaZlzLvjSFDzd5Ea/ttQokbI
# YViY9XwCFjyDKK05huzUtw1T0PhH5nUwjewwk3YUpltLXXRhTT8SkXbev1jLchAp
# QfDVxW0mdmgRQRNYmtwmKwH0iU1Z23jPgUo+QEdfyYFQc4UQIyFZYIpkVMHMIRro
# OBl8ZhzNeDhFMJlP/2NPTLuqDQhTQXxYPUez+rbsjDIJAsxsPAxWEQIDAQABo4IB
# WTCCAVUwEgYDVR0TAQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUaDfg67Y7+F8Rhvv+
# YXsIiGX0TkIwHwYDVR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0P
# AQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHcGCCsGAQUFBwEBBGswaTAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAC
# hjVodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9v
# dEc0LmNydDBDBgNVHR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5j
# b20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNybDAcBgNVHSAEFTATMAcGBWeBDAED
# MAgGBmeBDAEEATANBgkqhkiG9w0BAQwFAAOCAgEAOiNEPY0Idu6PvDqZ01bgAhql
# +Eg08yy25nRm95RysQDKr2wwJxMSnpBEn0v9nqN8JtU3vDpdSG2V1T9J9Ce7FoFF
# UP2cvbaF4HZ+N3HLIvdaqpDP9ZNq4+sg0dVQeYiaiorBtr2hSBh+3NiAGhEZGM1h
# mYFW9snjdufE5BtfQ/g+lP92OT2e1JnPSt0o618moZVYSNUa/tcnP/2Q0XaG3Ryw
# YFzzDaju4ImhvTnhOE7abrs2nfvlIVNaw8rpavGiPttDuDPITzgUkpn13c5Ubdld
# AhQfQDN8A+KVssIhdXNSy0bYxDQcoqVLjc1vdjcshT8azibpGL6QB7BDf5WIIIJw
# 8MzK7/0pNVwfiThV9zeKiwmhywvpMRr/LhlcOXHhvpynCgbWJme3kuZOX956rEnP
# LqR0kq3bPKSchh/jwVYbKyP/j7XqiHtwa+aguv06P0WmxOgWkVKLQcBIhEuWTatE
# QOON8BUozu3xGFYHKi8QxAwIZDwzj64ojDzLj4gLDb879M4ee47vtevLt/B3E+bn
# KD+sEq6lLyJsQfmCXBVmzGwOysWGw/YmMwwHS6DTBwJqakAwSEs0qFEgu60bhQji
# WQ1tygVQK+pKHJ6l/aCnHwZ05/LWUpD9r4VIIflXO7ScA+2GRfS0YW6/aOImYIbq
# yK+p/pQd52MbOoZWeE4wggdoMIIFUKADAgECAhAHm0XlRx2dupD0A91/gwV9MA0G
# CSqGSIb3DQEBCwUAMGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwg
# SW5jLjFBMD8GA1UEAxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcg
# UlNBNDA5NiBTSEEzODQgMjAyMSBDQTEwHhcNMjIwNzI4MDAwMDAwWhcNMjUwNjI4
# MjM1OTU5WjBtMQswCQYDVQQGEwJDSDEQMA4GA1UECBMHWnVlcmljaDEWMBQGA1UE
# BxMNQnJ1ZXR0aXNlbGxlbjESMBAGA1UEChMJT250cmV4IEFHMQwwCgYDVQQLEwND
# VE8xEjAQBgNVBAMTCU9udHJleCBBRzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCC
# AgoCggIBAN/Ghk02PiOCEsIguFhKkxRt9SeEPQjb+xroF9t34LHul33L6VDYNDIo
# U/WKjTUWT7r/7WqvIEWB+jufC0BYGX0zMm+JW1eJvFWaQ+koKAm3TPY2M2AXkO4C
# KN3+rGAT4DOrC1zyT7w3YI4D9BkrAHd/2JU/NbvUQl/fjobALG4/36PMPGJmzE3y
# q+QKgImHBaZ7CsJ5hrFIuMBaiXoqH9gduSdd9sD986s+3VfzQMi5QrYnnYiJqRTL
# dABCufDQEOgn0R3gma2vrVaQlKXHKP91sGp2lCdNE1F61lpKKzzwAiN4OWhBxlNK
# 19xIg2OV90g53l2BlOK0sG2YVygfGZ/3Z5AF4Nbr/Ps4iVNY2lVW1OTBdpa4/1cO
# q0g4guOoCc8dwNnN99RUpMGlqHRfkc272w8Rzg1S3cwlqfvjie49lvlkK7HnlZxc
# SyI0bQnyk1vOZkPbXb903nmNehqM+BGGcBMKVfarB3XqCJFeedRaorK6gpGKz+Km
# fn4HmI3ENrd1crvhnae9KCOiaV6+F691f7OuyHnYYH2C3UhqMaUY4K4Y+GWlx6EY
# v0zAuKKoSAFNSYXZZipgvUlGSjpUeWw88uOJlJEv383g8DKMtIeguIu/SUgP7mSQ
# Xy525kSrZG3w+qK08q+lNrr6RmNxvSlhWQ+0NYbHsF8Vg66007KFAgMBAAGjggIG
# MIICAjAfBgNVHSMEGDAWgBRoN+Drtjv4XxGG+/5hewiIZfROQjAdBgNVHQ4EFgQU
# 1w/iYCFGy/sHJw0leNX7sZbgMbcwDgYDVR0PAQH/BAQDAgeAMBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMIG1BgNVHR8Ega0wgaowU6BRoE+GTWh0dHA6Ly9jcmwzLmRpZ2lj
# ZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWduaW5nUlNBNDA5NlNIQTM4
# NDIwMjFDQTEuY3JsMFOgUaBPhk1odHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGln
# aUNlcnRUcnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQyMDIxQ0ExLmNy
# bDA+BgNVHSAENzA1MDMGBmeBDAEEATApMCcGCCsGAQUFBwIBFhtodHRwOi8vd3d3
# LmRpZ2ljZXJ0LmNvbS9DUFMwgZQGCCsGAQUFBwEBBIGHMIGEMCQGCCsGAQUFBzAB
# hhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wXAYIKwYBBQUHMAKGUGh0dHA6Ly9j
# YWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWduaW5n
# UlNBNDA5NlNIQTM4NDIwMjFDQTEuY3J0MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcN
# AQELBQADggIBAFYUSriK8lc6QRH6PqMcx0y5trKycljYdsWRQVMAz+/CjVbQMJeq
# enUHOy3wM+RwwuEcyRrYsCXNxHat9ghruP0s0qSDgOEHeWjG1ObP0sCk5sdFJTSg
# EnegZziW5NKjs1L/TsMYjOqdxytXoOoNvTM7dGf/YYHZ7/+9N+KpyarXNYIuDiKW
# 8ayvW+npsEGTWy/DiPl8RG9u8h38kPddeK/Fv53YbK7P5iOgskQqAeJBMr3o5ICs
# MVLGpYHsH+pRj5wRirj55L6fwdexp5sH2kSS4NOJR3W+ZYMuFiIiJJQuwjL4/A46
# cqB3M0MjtGy1hEQCnA4hNdvEGRQN71rj/Paa83LPZHkLCOe/oziTnrTeihYcy2OM
# dzfWwreUCGqYYVV9AWwiZ99w6YucRVch94e0+7A02pvLExcCZT+YrTHU6REHGoCk
# zgaVmD4OoakmDt1sl5COvlpbeX79SkuJYBoIefySepRewZnsbZurq4qwnWm1amRE
# heVoX/ONr+oqGH+MruOq4OQQce89VkhWNukI4wSyGKYG6/koOQNK0umm354ds+AA
# 2jhBirUSAio/+SQga9sFlElfxRd8FbZXOEYgDdRcF2GtrAwXvvFBZtubfm7h74Hm
# 3Sy0ID82UKP6HdzbiPUkxl6nuDwiWU9DistUfzLOUBzWz8AzW0onnrdzMYIaUzCC
# Gk8CAQEwfTBpMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4x
# QTA/BgNVBAMTOERpZ2lDZXJ0IFRydXN0ZWQgRzQgQ29kZSBTaWduaW5nIFJTQTQw
# OTYgU0hBMzg0IDIwMjEgQ0ExAhAHm0XlRx2dupD0A91/gwV9MA0GCWCGSAFlAwQC
# AQUAoHwwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIE
# IFGumM9hr/8KHxZ5LFW1fHlJI9KAx6H/lrj/grB4JPh3MA0GCSqGSIb3DQEBAQUA
# BIICAJYg1j7Ree03S4FgWlPIvPwvc5U5jIM3JrTxw5xeqbSGpoxEVahXWzqCqAnk
# iBT7FHwXvh/Y7mp5uJfGS2Ls5oFKkaSqAsvDGo2yVlBah1sn0+cS8vB0LUvBHkYQ
# t6Y/sdCdF8WEgwywbzpUKclK8z6q4sUT0uZdgJ6hYw85ibV6AgH1ybshN+QOJ2Yw
# L5yqbQMF4PAGN4wUKzKiff4LCGP9ZhRSJt3b8X87RyBqzt4ArsdMwbi08kgfs/Sf
# Ut0RcHSpCr+BoSQNqhtEAUuwStZK3u/T6c7tTj1rUe4uShw84YQ3dw9F0Jpkrszq
# d1g6iMYL4r6DN65uBgBTvSvjvuGXN8dPp9DPqZwGhQr2VJJbk/ynDc1LB2xxQKUh
# 0PnSdkY8fni/auqxEYZ1C1NlcKLpLH+kmY69xX7Ejn0K95rHydn++7fwZnzQ8gHD
# e761q7YkPIdTXl5eqTyGbYSnTQSD+puGV7eVQWFCXHdN+YyJUNbZuWI4XQQ7YchF
# HB++jKdI6F6YJref3S+UbrI9yZEtJ/+wnVQEW9p97Ov01RWqWo3Znai/O8KtEw1f
# ZKrqv+rmMc3R0DGILw582fiIFPB+S9J/H12qUQePWC/K69PA8O4MZxGGxLLk2RMA
# bIbOHttpxsGPMg3A2mtVGpD3E25JAvfOM9tSUT6uW0OLTDBsoYIXKTCCFyUGCisG
# AQQBgjcDAwExghcVMIIXEQYJKoZIhvcNAQcCoIIXAjCCFv4CAQMxDzANBglghkgB
# ZQMEAgEFADBnBgsqhkiG9w0BCRABBKBYBFYwVAIBAQYJYIZIAYb9bAcBMCEwCQYF
# Kw4DAhoFAAQUj82C1IDx9XD/3tXvy6AhcTzm1TcCEC+5pi2YLd415yIkV/4oiJgY
# DzIwMjUwMjEzMTQxMzIxWqCCEwMwgga8MIIEpKADAgECAhALrma8Wrp/lYfG+ekE
# 4zMEMA0GCSqGSIb3DQEBCwUAMGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdp
# Q2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2
# IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0EwHhcNMjQwOTI2MDAwMDAwWhcNMzUxMTI1
# MjM1OTU5WjBCMQswCQYDVQQGEwJVUzERMA8GA1UEChMIRGlnaUNlcnQxIDAeBgNV
# BAMTF0RpZ2lDZXJ0IFRpbWVzdGFtcCAyMDI0MIICIjANBgkqhkiG9w0BAQEFAAOC
# Ag8AMIICCgKCAgEAvmpzn/aVIauWMLpbbeZZo7Xo/ZEfGMSIO2qZ46XB/QowIEMS
# vgjEdEZ3v4vrrTHleW1JWGErrjOL0J4L0HqVR1czSzvUQ5xF7z4IQmn7dHY7yijv
# oQ7ujm0u6yXF2v1CrzZopykD07/9fpAT4BxpT9vJoJqAsP8YuhRvflJ9YeHjes4f
# duksTHulntq9WelRWY++TFPxzZrbILRYynyEy7rS1lHQKFpXvo2GePfsMRhNf1F4
# 1nyEg5h7iOXv+vjX0K8RhUisfqw3TTLHj1uhS66YX2LZPxS4oaf33rp9HlfqSBeP
# ejlYeEdU740GKQM7SaVSH3TbBL8R6HwX9QVpGnXPlKdE4fBIn5BBFnV+KwPxRNUN
# K6lYk2y1WSKour4hJN0SMkoaNV8hyyADiX1xuTxKaXN12HgR+8WulU2d6zhzXomJ
# 2PleI9V2yfmfXSPGYanGgxzqI+ShoOGLomMd3mJt92nm7Mheng/TBeSA2z4I78Jp
# wGpTRHiT7yHqBiV2ngUIyCtd0pZ8zg3S7bk4QC4RrcnKJ3FbjyPAGogmoiZ33c1H
# G93Vp6lJ415ERcC7bFQMRbxqrMVANiav1k425zYyFMyLNyE1QulQSgDpW9rtvVcI
# H7WvG9sqYup9j8z9J1XqbBZPJ5XLln8mS8wWmdDLnBHXgYly/p1DhoQo5fkCAwEA
# AaOCAYswggGHMA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBYGA1UdJQEB
# /wQMMAoGCCsGAQUFBwMIMCAGA1UdIAQZMBcwCAYGZ4EMAQQCMAsGCWCGSAGG/WwH
# ATAfBgNVHSMEGDAWgBS6FtltTYUvcyl2mi91jGogj57IbzAdBgNVHQ4EFgQUn1cs
# A3cOKBWQZqVjXu5Pkh92oFswWgYDVR0fBFMwUTBPoE2gS4ZJaHR0cDovL2NybDMu
# ZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0UlNBNDA5NlNIQTI1NlRpbWVT
# dGFtcGluZ0NBLmNybDCBkAYIKwYBBQUHAQEEgYMwgYAwJAYIKwYBBQUHMAGGGGh0
# dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBYBggrBgEFBQcwAoZMaHR0cDovL2NhY2Vy
# dHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0UlNBNDA5NlNIQTI1NlRp
# bWVTdGFtcGluZ0NBLmNydDANBgkqhkiG9w0BAQsFAAOCAgEAPa0eH3aZW+M4hBJH
# 2UOR9hHbm04IHdEoT8/T3HuBSyZeq3jSi5GXeWP7xCKhVireKCnCs+8GZl2uVYFv
# Qe+pPTScVJeCZSsMo1JCoZN2mMew/L4tpqVNbSpWO9QGFwfMEy60HofN6V51sMLM
# XNTLfhVqs+e8haupWiArSozyAmGH/6oMQAh078qRh6wvJNU6gnh5OruCP1QUAvVS
# u4kqVOcJVozZR5RRb/zPd++PGE3qF1P3xWvYViUJLsxtvge/mzA75oBfFZSbdakH
# Je2BVDGIGVNVjOp8sNt70+kEoMF+T6tptMUNlehSR7vM+C13v9+9ZOUKzfRUAYSy
# yEmYtsnpltD/GWX8eM70ls1V6QG/ZOB6b6Yum1HvIiulqJ1Elesj5TMHq8CWT/xr
# W7twipXTJ5/i5pkU5E16RSBAdOp12aw8IQhhA/vEbFkEiF2abhuFixUDobZaA0Vh
# qAsMHOmaT3XThZDNi5U2zHKhUs5uHHdG6BoQau75KiNbh0c+hatSF+02kULkftAR
# jsyEpHKsF7u5zKRbt5oK5YGwFvgc4pEVUNytmB3BpIiowOIIuDgP5M9WArHYSAR1
# 6gc0dP2XdkMEP5eBsX7bf/MGN4K3HP50v/01ZHo/Z5lGLvNwQ7XHBx1yomzLP8lx
# 4Q1zZKDyHcp4VQJLu2kWTsKsOqQwggauMIIElqADAgECAhAHNje3JFR82Ees/Shm
# Kl5bMA0GCSqGSIb3DQEBCwUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERp
# Z2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIy
# MzU5NTlaMGMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7
# MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1l
# U3RhbXBpbmcgQ0EwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUG
# SbPBPXJJUVXHJQPE8pE3qZdRodbSg9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOc
# iQt/nR+eDzMfUBMLJnOWbfhXqAJ9/UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkr
# PkLcZ47qUT3w1lbU5ygt69OxtXXnHwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rw
# N3mfXazL6IRktFLydkf3YYMZ3V+0VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSm
# xR3NNg1c1eYbqMFkdECnwHLFuk4fsbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu
# 9Yemj052FVUmcJgmf6AaRyBD40NjgHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirH
# kr+g3uM+onP65x9abJTyUpURK1h0QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506
# o9UD4L/wojzKQtwYSH8UNM/STKvvmz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklN
# iyDSLFc1eSuo80VgvCONWPfcYd6T/jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGT
# yYwMO1uKIqjBJgj5FBASA31fI7tk42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgA
# DoRU7s7pXcheMBK9Rp6103a50g5rmQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0T
# AQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYD
# VR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMIMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYY
# aHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2Fj
# ZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNV
# HR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRU
# cnVzdGVkUm9vdEc0LmNybDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1s
# BwEwDQYJKoZIhvcNAQELBQADggIBAH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPP
# MFPOvxj7x1Bd4ksp+3CKDaopafxpwc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKW
# b8RQTGIdDAiCqBa9qVbPFXONASIlzpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpP
# kWaeLJ7giqzl/Yy8ZCaHbJK9nXzQcAp876i8dU+6WvepELJd6f8oVInw1YpxdmXa
# zPByoyP6wCeCRK6ZJxurJB4mwbfeKuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKv
# xMfBwWpx2cYTgAnEtp/Nh4cku0+jSbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl6
# 3f8lY5knLD0/a6fxZsNBzU+2QJshIUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YB
# T70/O3itTK37xJV77QpfMzmHQXh6OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4n
# LCbbbxV7HhmLNriT1ObyF5lZynDwN7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvt
# lUG3OtUVmDG0YgkPCr2B2RP+v6TR81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm
# 2fBldkKmKYcJRyvmfxqkhQ/8mJb2VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqh
# K/bt1nz8MIIFjTCCBHWgAwIBAgIQDpsYjvnQLefv21DiCEAYWjANBgkqhkiG9w0B
# AQwFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVk
# IElEIFJvb3QgQ0EwHhcNMjIwODAxMDAwMDAwWhcNMzExMTA5MjM1OTU5WjBiMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQw
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQC/5pBzaN675F1KPDAiMGkz
# 7MKnJS7JIT3yithZwuEppz1Yq3aaza57G4QNxDAf8xukOBbrVsaXbR2rsnnyyhHS
# 5F/WBTxSD1Ifxp4VpX6+n6lXFllVcq9ok3DCsrp1mWpzMpTREEQQLt+C8weE5nQ7
# bXHiLQwb7iDVySAdYyktzuxeTsiT+CFhmzTrBcZe7FsavOvJz82sNEBfsXpm7nfI
# SKhmV1efVFiODCu3T6cw2Vbuyntd463JT17lNecxy9qTXtyOj4DatpGYQJB5w3jH
# trHEtWoYOAMQjdjUN6QuBX2I9YI+EJFwq1WCQTLX2wRzKm6RAXwhTNS8rhsDdV14
# Ztk6MUSaM0C/CNdaSaTC5qmgZ92kJ7yhTzm1EVgX9yRcRo9k98FpiHaYdj1ZXUJ2
# h4mXaXpI8OCiEhtmmnTK3kse5w5jrubU75KSOp493ADkRSWJtppEGSt+wJS00mFt
# 6zPZxd9LBADMfRyVw4/3IbKyEbe7f/LVjHAsQWCqsWMYRJUadmJ+9oCw++hkpjPR
# iQfhvbfmQ6QYuKZ3AeEPlAwhHbJUKSWJbOUOUlFHdL4mrLZBdd56rF+NP8m800ER
# ElvlEFDrMcXKchYiCd98THU/Y+whX8QgUWtvsauGi0/C1kVfnSD8oR7FwI+isX4K
# Jpn15GkvmB0t9dmpsh3lGwIDAQABo4IBOjCCATYwDwYDVR0TAQH/BAUwAwEB/zAd
# BgNVHQ4EFgQU7NfjgtJxXWRM3y5nP+e6mK4cD08wHwYDVR0jBBgwFoAUReuir/SS
# y4IxLVGLp6chnfNtyA8wDgYDVR0PAQH/BAQDAgGGMHkGCCsGAQUFBwEBBG0wazAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAC
# hjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURS
# b290Q0EuY3J0MEUGA1UdHwQ+MDwwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwEQYDVR0gBAowCDAGBgRV
# HSAAMA0GCSqGSIb3DQEBDAUAA4IBAQBwoL9DXFXnOF+go3QbPbYW1/e/Vwe9mqyh
# hyzshV6pGrsi+IcaaVQi7aSId229GhT0E0p6Ly23OO/0/4C5+KH38nLeJLxSA8hO
# 0Cre+i1Wz/n096wwepqLsl7Uz9FDRJtDIeuWcqFItJnLnU+nBgMTdydE1Od/6Fmo
# 8L8vC6bp8jQ87PcDx4eo0kxAGTVGamlUsLihVo7spNU96LHc/RzY9HdaXFSMb++h
# UD38dglohJ9vytsgjTVgHAIDyyCwrFigDkBjxZgiwbJZ9VVrzyerbHbObyMt9H5x
# aiNrIv8SuFQtJ37YOtnwtoeW/VvRXKwYw02fc7cBqZ9Xql4o4rmUMYIDdjCCA3IC
# AQEwdzBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5
# BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0
# YW1waW5nIENBAhALrma8Wrp/lYfG+ekE4zMEMA0GCWCGSAFlAwQCAQUAoIHRMBoG
# CSqGSIb3DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMjEz
# MTQxMzIxWjArBgsqhkiG9w0BCRACDDEcMBowGDAWBBTb04XuYtvSPnvk9nFIUIck
# 1YZbRTAvBgkqhkiG9w0BCQQxIgQgA3fVWtQBbvKdyrAMjg++AC+KgfJzcy15r4Gp
# ncPi49QwNwYLKoZIhvcNAQkQAi8xKDAmMCQwIgQgdnafqPJjLx9DCzojMK7WVnX+
# 13PbBdZluQWTmEOPmtswDQYJKoZIhvcNAQEBBQAEggIAJV/u3EkvmQikS0jufjX/
# UNytlrTtXgHz5pgNbPVLKhd7RgeU9n1GqurAM1jWQUHcGy5HXtoEZNvqhDEFl3Mr
# hdFdpDHKPPjIZeuHvi1K3RcYNfrhro6zCGyblVkHfX194EvhU1FLYqfMLvJWRHjI
# syNXgHy3sENCUuaOQ28/Bq3oJeQTbnwip/zDvC+ACKuLhwPorrJu0x9sHn+JVi7Q
# IzfPRk5bKOMHoFKI4sTjG4rmFBteDM+SoxoNHNb2ILUSfHy0S+5BpfgeH6O3rJ1f
# FVk1gQi+mRlG1P0kQynVkriVcGJP/4L+L7neixU85uMZha5ZnszV08Kv6i3KUxqm
# oSzhwgQACq4U2qt9EeVYyg2S+LiepUUQr8L1xSxA5x04jwGin9Pm+Sn4uqeWoVKs
# 02y+ffHUee/Tnl0kQTbZxckCAeA0ubnHso2+iZ8xfjQMFjpX/DTw8m7h4V06U0d4
# iR8oOg1GGnF2pfXJfg+5PiemRh7Z8BbYNQSS+u7OT5uBJ09n3fDWRHyRJueEkQ+q
# 0YiEQtCamIw/cMFVphNe/QtzFpgrm6jxq1+SzwWC/t2tWbTQ8mXKE6DEIjn2h1EI
# ICp0lVJUXYZfTBxAfA3nEU5ZxrF0Yoa5GfVvHTFGl4BKW4h0Z3RGySCJgDqcSDs5
# zNNlztAKkjoLVptvz1XJ/sk=
# SIG # End signature block
